/*
 * SetupAPI device installer
 *
 * Copyright 2000 Andreas Mohr for CodeWeavers
 * Copyright 2004 Aric Stewart for CodeWeavers
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdarg.h>

#include "windef.h"
#include "winbase.h"
#include "wingdi.h"
#include "winuser.h"
#include "winnls.h"
#include "winreg.h"
#include "setupapi.h"
#include "wine/debug.h"
#include "wine/unicode.h"
#include "winioctl.h"
#include "dbt.h"

#include "setupapi_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(setupapi);

/***********************************************************************
 *		SetupDiGetDeviceInterfaceDetailA (SETUPAPI.@)
 */
BOOL WINAPI SetupDiGetDeviceInterfaceDetailA(
      HDEVINFO DeviceInfoSet,
      PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData,
      PSP_DEVICE_INTERFACE_DETAIL_DATA_A DeviceInterfaceDetailData,
      DWORD DeviceInterfaceDetailDataSize,
      PDWORD RequiredSize,
      PSP_DEVINFO_DATA DeviceInfoData )
{

   devnode *node = (devnode*)DeviceInterfaceData->Reserved;

    FIXME("%p %p %p %li %p %p\n",DeviceInfoSet, DeviceInterfaceData,
            DeviceInterfaceDetailData, DeviceInterfaceDetailDataSize, 
            RequiredSize, DeviceInfoData);

    node->registry_fn(node, SPDRP_DEVICE_PATH, NULL,
                    (LPBYTE)DeviceInterfaceDetailData->DevicePath,
                    DeviceInterfaceDetailDataSize, RequiredSize, FALSE);

    return TRUE;
}

/***********************************************************************
 *		SetupDiGetDeviceInterfaceDetailW (SETUPAPI.@)
 */
BOOL WINAPI SetupDiGetDeviceInterfaceDetailW(
      HDEVINFO DeviceInfoSet,
      PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData,
      PSP_DEVICE_INTERFACE_DETAIL_DATA_W DeviceInterfaceDetailData,
      DWORD DeviceInterfaceDetailDataSize,
      PDWORD RequiredSize,
      PSP_DEVINFO_DATA DeviceInfoData )
{
    FIXME("\n");
    return FALSE;
}

/***********************************************************************
 *		SetupDiEnumDeviceInterfaces (SETUPAPI.@)
 */
BOOL WINAPI SetupDiEnumDeviceInterfaces(
       HDEVINFO DeviceInfoSet,
       PSP_DEVINFO_DATA DeviceInfoData,
       CONST GUID * InterfaceClassGuid,
       DWORD MemberIndex,
       PSP_DEVICE_INTERFACE_DATA DeviceInterfaceData)
{
    devinfo_struct *tree = (devinfo_struct*)DeviceInfoSet;

    TRACE("(%p, %p, %s, %li %p\n",DeviceInfoSet, DeviceInfoData,
           debugstr_guid(InterfaceClassGuid), MemberIndex, DeviceInterfaceData);

    if (!tree)
        return FALSE;

    if (MemberIndex >= tree->count)
    {
        SetLastError(ERROR_NO_MORE_ITEMS);
        return FALSE; 
    }

    memcpy(&(DeviceInterfaceData->InterfaceClassGuid),
           &(tree->devices[MemberIndex].guid),sizeof(GUID));
    DeviceInterfaceData->Flags = tree->devices[MemberIndex].flags;
    DeviceInterfaceData->Reserved = (ULONG_PTR)&(tree->devices[MemberIndex]);

    return TRUE;
}

/***********************************************************************
 *		SetupDiGetClassDevsA (SETUPAPI.@)
 */
HDEVINFO WINAPI SetupDiGetClassDevsA(
       CONST GUID *class,
       LPCSTR enumstr,
       HWND parent,
       DWORD flags)
{
    devinfo_struct *info;

    TRACE("%s %s %p %08lx\n",debugstr_guid(class),enumstr,parent,flags);

    /* TODO: make this thread safe*/
    info = HeapAlloc(GetProcessHeap(),0,sizeof(devinfo_struct));
    memset(info,0,sizeof(devinfo_struct));
    info->flags = flags;
    info->unicode = FALSE;

    USB_GetDeviceDevs(info,class,enumstr);

    if (info->count > 0)
        return (HDEVINFO)info;
    else
        HeapFree(GetProcessHeap(),0,info);

    return (HDEVINFO) INVALID_HANDLE_VALUE;
}

/***********************************************************************
 *		SetupDiGetClassDevsW (SETUPAPI.@)
 */
HDEVINFO WINAPI SetupDiGetClassDevsW(
       CONST GUID *class,
       LPCWSTR enumstr,
       HWND parent,
       DWORD flags)
{
    FIXME("%s %s %p %08lx\n",debugstr_guid(class),debugstr_w(enumstr),parent,flags);

    return (HDEVINFO) INVALID_HANDLE_VALUE;
}
/***********************************************************************
 *		SetupDiEnumDeviceInfo (SETUPAPI.@)
 */
BOOL WINAPI SetupDiEnumDeviceInfo(
        HDEVINFO  devinfo,
        DWORD  index,
        PSP_DEVINFO_DATA info)
{
    DWORD rc;
    devinfo_struct *tree = (devinfo_struct*)devinfo;

    TRACE("%p %ld %p\n", devinfo, index, info );

    if(info==NULL)
        return FALSE;
    if(info->cbSize < sizeof(*info))
        return FALSE;
    if (tree == NULL)
        return FALSE;

    if (index >= tree->count)
    {
        SetLastError(ERROR_NO_MORE_ITEMS);
        return FALSE; 
    }

    memcpy(&(info->ClassGuid),&(tree->devices[index].guid),sizeof(GUID));
    info->Reserved = (ULONG_PTR)&(tree->devices[index]);

    /* 
     * Used by cfgmgr32... this is BAD BAD not threadsafe 
     * also it should be a handle that can be used to retreave data like this
     * not the streight up data.... but this hack works for the ipod
     */
    tree->devices[index].registry_fn(&(tree->devices[index]), SPDRP_DEVICE_ID,
                                     NULL, NULL, 0, &rc, FALSE);
    info->DevInst = (DWORD)(tree->devices[index].deviceId); 

    return TRUE;
}

/***********************************************************************
 *		SetupDiDestroyDeviceInfoList (SETUPAPI.@)
 */
BOOL WINAPI SetupDiDestroyDeviceInfoList( HDEVINFO devinfo )
{
    devinfo_struct *info = (devinfo_struct*)devinfo;

    TRACE("%p\n", devinfo);

    if (!info)
        return TRUE;

    if (info->count > 0)
    {
        int i;
        for (i = 0; i < info->count; i++)
        {
            if (info->devices[i].hardwareid)
                HeapFree(GetProcessHeap(),0,info->devices[i].hardwareid);
        }
        HeapFree(GetProcessHeap(),0,info->devices);
    }

    HeapFree(GetProcessHeap(),0,info);

    return TRUE;
}

/***********************************************************************
 *		SetupDiGetDeviceRegistryPropertyA (SETUPAPI.@)
 */
BOOL WINAPI SetupDiGetDeviceRegistryPropertyA(
        HDEVINFO  devinfo,
        PSP_DEVINFO_DATA  DeviceInfoData,
        DWORD   Property,
        PDWORD  PropertyRegDataType,
        PBYTE   PropertyBuffer,
        DWORD   PropertyBufferSize,
        PDWORD  RequiredSize)
{
    devnode *node = (devnode*)DeviceInfoData->Reserved;

    TRACE("%p %p %ld %p %p %ld %p\n", devinfo, DeviceInfoData,
        Property, PropertyRegDataType, PropertyBuffer, PropertyBufferSize,
        RequiredSize);

    if (!node)
        return FALSE;

    return (BOOL)node->registry_fn(node, Property, PropertyRegDataType,
                    PropertyBuffer, PropertyBufferSize, RequiredSize, FALSE);
}

BOOL WINAPI SetupDiSetClassInstallParamsA( HDEVINFO  DeviceInfoSet,
  PSP_DEVINFO_DATA  DeviceInfoData, PSP_CLASSINSTALL_HEADER  ClassInstallParams,
  DWORD ClassInstallParamsSize)
{
    FIXME("STUB: %p, %p, %lx, %lu\n",DeviceInfoSet, DeviceInfoData,
ClassInstallParams->InstallFunction, ClassInstallParamsSize);

    return FALSE;
}

BOOL WINAPI SetupDiCallClassInstaller( DWORD InstallFunction, HDEVINFO
DeviceInfoSet, PSP_DEVINFO_DATA  DeviceInfoData)
{
    FIXME("STUB\n");
    return FALSE;
}

BOOL WINAPI SetupDiGetDeviceInstallParamsA( HDEVINFO  DeviceInfoSet,
    PSP_DEVINFO_DATA  DeviceInfoData, PSP_DEVINSTALL_PARAMSA
    DeviceInstallParams)
{
    FIXME("STUB\n");
    return FALSE;
}

HKEY WINAPI SetupDiOpenDevRegKey( HDEVINFO DeviceInfoSet,
    PSP_DEVINFO_DATA DeviceInfoData, DWORD Scope, DWORD HwProfile, DWORD KeyType,
    REGSAM samDesired)
{
    FIXME("STUB\n");
    return INVALID_HANDLE_VALUE;
}
