
#include "wine/test.h"
#include "winbase.h"
#include "winreg.h"

#include "pshpack1.h"

typedef struct {
	WORD cLength;
	WORD nVersion;
	BOOL fFullPathTitle:1;
	BOOL fSaveLocalView:1;
	BOOL fNotShell:1;
	BOOL fSimpleDefault:1;
	BOOL fDontShowDescBar:1;
	BOOL fNewWindowMode:1;
	BOOL fShowCompColor:1;
	BOOL fDontPrettyNames:1;
	BOOL fAdminsCreateCommonGroups:1;
	UINT fUnusedFlags:7;
	UINT fMenuEnumFilter;
} CABINETSTATE ;

#include "poppack.h"

typedef DWORD (WINAPI *fnReadCabinetState)(CABINETSTATE *, int);
fnReadCabinetState ReadCabinetState;

typedef DWORD (WINAPI *fnWriteCabinetState)(CABINETSTATE *);
fnWriteCabinetState WriteCabinetState;

#define get_fn(x,ord) x = (fn##x) \
		GetProcAddress(hShell32, (LPCSTR) (ord)); \
		if(!(x)) { return FALSE; }

static BOOL resolve(void)
{
	HMODULE hShell32 = LoadLibrary("shell32.dll");

	if(!hShell32)
		return FALSE;

	get_fn(ReadCabinetState,654)
	get_fn(WriteCabinetState,652)

	return TRUE;
}

static LPCSTR szCab = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\CabinetState";

START_TEST(cabinet)
{
	CABINETSTATE cab_state, *cs;
	DWORD r, size, type;
	HKEY hkey;
	BYTE buffer[0x20];

	if( !resolve() )
		return;

	/* set things up if they aren't set up already */
	ReadCabinetState(&cab_state, sizeof cab_state);
	WriteCabinetState(&cab_state);

	r = ReadCabinetState(&cab_state, sizeof cab_state);
	ok(r, "ReadCabinetState failed\n");

	r = RegOpenKey(HKEY_CURRENT_USER, szCab, &hkey);
	ok(r==ERROR_SUCCESS, "Failed to open hkey\n");

        /* read the registry */
	memset(buffer, 0, sizeof buffer);
	cs = (CABINETSTATE*)buffer;
	type = REG_BINARY;
	size = sizeof buffer;
	r = RegQueryValueEx(hkey, "Settings", NULL, &type, buffer, &size);
	ok(r==ERROR_SUCCESS, "Failed to read value\n");

	ok(!memcmp(buffer, &cab_state, sizeof cab_state),"Differs\n");

	r = RegCloseKey(hkey);
        ok(r == ERROR_SUCCESS, "couldn't close key\n");

	r = ReadCabinetState(&cab_state, 0);
	ok(!r, "zero length... succeeded!\n");

	r = ReadCabinetState(NULL, sizeof cab_state);
	ok(!r, "NULL pointer... succeeded!\n");

	if( cab_state.fFullPathTitle )
		cab_state.fFullPathTitle = FALSE;
	else
		cab_state.fFullPathTitle = TRUE;

	r = WriteCabinetState(&cab_state);
	ok(r,"WriteCabinetState failed\n");

	r = ReadCabinetState(cs, sizeof *cs);
	ok(r, "ReadCabinetState failed\n");

	ok(!memcmp(buffer, &cab_state, sizeof cab_state), "Differs\n");
}

